==> mxnet_model_server-0.4b1-py2.py3-none-any.whl: mms/model_service/model_service.py <==
# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#     http://www.apache.org/licenses/LICENSE-2.0
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

"""`ModelService` defines an API for base model service.
"""
# pylint: disable=W0223

import os
import sys
import time
from abc import ABCMeta, abstractmethod, abstractproperty

from mms.log import get_logger
from mms.metrics_manager import MetricsManager

logger = get_logger()
URL_PREFIX = ('http://', 'https://', 's3://')


class ModelService(object):
    """
    ModelService wraps up all preprocessing, inference and postprocessing
    functions used by model service. It is defined in a flexible manner to
    be easily extended to support different frameworks.
    """
    __metaclass__ = ABCMeta

    def __init__(self, model_name, model_dir, manifest, gpu=None):  # pylint: disable=unused-argument
        self.ctx = None

    @abstractmethod
    def inference(self, data):
        """
        Wrapper function to run pre-process, inference and post-process functions.

        Parameters
        ----------
        data : list of object
            Raw input from request.

        Returns
        -------
        list of outputs to be sent back to client.
            data to be sent back
        """
        pass

    @abstractmethod
    def ping(self):
        """
        Ping to get system's health.

        Returns
        -------
        String
            A message, "health": "healthy!", to show system is healthy.
        """
        pass

    @abstractproperty
    def signature(self):
        """
        Signiture for model service.

        Returns
        -------
        Dict
            Model service signature.
        """
        pass


class SingleNodeService(ModelService):
    """
    SingleNodeModel defines abstraction for model service which loads a
    single model.
    """

    def inference(self, data):
        """
        Wrapper function to run preprocess, inference and postprocess functions.

        Parameters
        ----------
        data : list of object
            Raw input from request.

        Returns
        -------
        list of outputs to be sent back to client.
            data to be sent back
        """
        pre_start_time = time.time()
        data = self._preprocess(data)
        infer_start_time = time.time()
        data = self._inference(data)
        post_start_ms = time.time()
        data = self._postprocess(data)
        post_end_ms = time.time()

        pre_time_in_ms = (infer_start_time - pre_start_time) * 1000
        if self.model_name + '_LatencyPreprocess' in MetricsManager.metrics:
            MetricsManager.metrics[self.model_name + '_LatencyPreprocess'].update(pre_time_in_ms)

        infer_time_in_ms = (post_start_ms - infer_start_time) * 1000
        if self.model_name + '_LatencyInference' in MetricsManager.metrics:
            MetricsManager.metrics[self.model_name + '_LatencyInference'].update(infer_time_in_ms)

        post_time_in_ms = (post_end_ms - post_start_ms) * 1000
        if self.model_name + '_LatencyPostprocess' in MetricsManager.metrics:
            MetricsManager.metrics[self.model_name + '_LatencyPostprocess'].update(post_time_in_ms)

        if self.model_name + '_LatencyOverall' in MetricsManager.metrics:
            MetricsManager.metrics[self.model_name + '_LatencyOverall'].update(pre_time_in_ms + infer_time_in_ms +
                                                                               post_time_in_ms)
        return data

    @abstractmethod
    def _inference(self, data):
        """
        Internal inference methods. Run forward computation and
        return output.

        Parameters
        ----------
        data : list of NDArray
            Preprocessed inputs in NDArray format.

        Returns
        -------
        list of NDArray
            Inference output.
        """
        return data

    def _preprocess(self, data):
        """
        Internal preprocess methods. Do transformation on raw
        inputs and convert them to NDArray.

        Parameters
        ----------
        data : list of object
            Raw inputs from request.

        Returns
        -------
        list of NDArray
            Processed inputs in NDArray format.
        """
        return data

    def _postprocess(self, data):
        """
        Internal postprocess methods. Do transformation on inference output
        and convert them to MIME type objects.

        Parameters
        ----------
        data : list of NDArray
            Inference output.

        Returns
        -------
        list of object
            list of outputs to be sent back.
        """
        return data


class MultiNodesService(ModelService):
    pass


def load_service(path, name=None):
    """
    Load the model-service into memory and associate it with each flask app worker
    :param path:
    :param name:
    :return:
    """
    try:
        if not name:
            name = os.path.splitext(os.path.basename(path))[0]

        module = None
        if sys.version_info[0] > 2:
            import importlib
            spec = importlib.util.spec_from_file_location(name, path)
            module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(module)

        else:
            import imp
            module = imp.load_source(name, path)

        return module
    except Exception:
        raise Exception('Incorrect or missing service file: ' + path)

==> mxnet_model_server-1.0b20181010-py2.py3-none-any.whl: mms/model_service/model_service.py <==
# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#     http://www.apache.org/licenses/LICENSE-2.0
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

"""`ModelService` defines an API for base model service.
"""
# pylint: disable=W0223

import ast
import json
import logging
import os
import time
from abc import ABCMeta, abstractmethod


class ModelService(object):
    """
    ModelService wraps up all preprocessing, inference and postprocessing
    functions used by model service. It is defined in a flexible manner to
    be easily extended to support different frameworks.
    """
    __metaclass__ = ABCMeta

    # noinspection PyUnusedLocal
    def __init__(self, model_name, model_dir, manifest, gpu=None):  # pylint: disable=unused-argument
        self.ctx = None
        self._context = None
        self._signature = None

    def initialize(self, context):
        """
        Internal initialize ModelService.

        :param context: MMS context object
        :return:
        """
        self._context = context
        properties = context.system_properties
        model_dir = properties.get("model_dir")

        signature_file_path = os.path.join(model_dir, context.manifest['Model']['Signature'])
        if not os.path.isfile(signature_file_path):
            raise ValueError("Signature file is not found.")

        with open(signature_file_path) as f:
            self._signature = json.load(f)

    @abstractmethod
    def inference(self, data):
        """
        Wrapper function to run pre-process, inference and post-process functions.

        Parameters
        ----------
        data : list of object
            Raw input from request.

        Returns
        -------
        list of outputs to be sent back to client.
            data to be sent back
        """
        pass

    @abstractmethod
    def ping(self):
        """
        Ping to get system's health.

        Returns
        -------
        String
            A message, "health": "healthy!", to show system is healthy.
        """
        pass

    def signature(self):
        """
        Signature for model service.

        Returns
        -------
        Dict
            Model service signature.
        """
        return self._signature

    # noinspection PyUnusedLocal
    def handle(self, data, context):  # pylint: disable=unused-argument
        """
        Backward compatible handle function.

        :param data:
        :param context:
        :return:

        """
        input_type = self._signature['input_type']

        input_data = []
        data_name = self._signature["inputs"][0]["data_name"]
        form_data = data[0].get(data_name)
        if form_data is None:
            form_data = data[0].get("body")

        if form_data is None:
            form_data = data[0].get("data")

        if input_type == "application/json":
            # user might not send content in HTTP request
            if isinstance(form_data, (bytes, bytearray)):
                form_data = ast.literal_eval(form_data.decode())

        input_data.append(form_data)

        ret = self.inference(input_data)
        if isinstance(ret, list):
            return ret

        return [ret]


class SingleNodeService(ModelService):
    """
    SingleNodeModel defines abstraction for model service which loads a
    single model.
    """

    def inference(self, data):
        """
        Wrapper function to run preprocess, inference and postprocess functions.

        Parameters
        ----------
        data : list of object
            Raw input from request.

        Returns
        -------
        list of outputs to be sent back to client.
            data to be sent back
        """
        preprocess_start = time.time()
        data = self._preprocess(data)
        inference_start = time.time()
        data = self._inference(data)
        postprocess_start = time.time()
        data = self._postprocess(data)
        end_time = time.time()

        logging.info("preprocess time: %.2f", (inference_start - preprocess_start) * 1000)
        logging.info("inference time: %.2f", (postprocess_start - inference_start) * 1000)
        logging.info("postprocess time: %.2f", (end_time - postprocess_start) * 1000)

        return data

    @abstractmethod
    def _inference(self, data):
        """
        Internal inference methods. Run forward computation and
        return output.

        Parameters
        ----------
        data : list of NDArray
            Preprocessed inputs in NDArray format.

        Returns
        -------
        list of NDArray
            Inference output.
        """
        return data

    def _preprocess(self, data):
        """
        Internal preprocess methods. Do transformation on raw
        inputs and convert them to NDArray.

        Parameters
        ----------
        data : list of object
            Raw inputs from request.

        Returns
        -------
        list of NDArray
            Processed inputs in NDArray format.
        """
        return data

    def _postprocess(self, data):
        """
        Internal postprocess methods. Do transformation on inference output
        and convert them to MIME type objects.

        Parameters
        ----------
        data : list of NDArray
            Inference output.

        Returns
        -------
        list of object
            list of outputs to be sent back.
        """
        return data
